const { Cc, Ci, Cr } = require("chrome");

var events = require("sdk/system/events");
var request = require("sdk/request").Request;
var p = require("sdk/page-mod");
var d = require("sdk/self").data;
var prefs = require("sdk/simple-prefs");
var ss = require("sdk/simple-storage");
var tabs = require("sdk/tabs");

var domains = {};
var commons = {};
var custom_mods = {};
var block_urls = {};


// Whitelist

var permanent_whitelist = {
	"*.plus.google.com":1,
	"*.hangouts.google.com":1,
	"*.clients5.google.com":1,
	"*.clients6.google.com":1,
	"*.arcot.com":1,
	"*.orteil.dashnet.org":1
};

if (!ss.storage.global_exclude_list)
	ss.storage.global_exclude_list = [];

for (var i in ss.storage.global_exclude_list)
	if (permanent_whitelist[ss.storage.global_exclude_list[i]])
		permanent_whitelist[ss.storage.global_exclude_list[i]] = 0;

for (var i in permanent_whitelist)
	if (permanent_whitelist[i] == 1)
		ss.storage.global_exclude_list.push(i);

delete permanent_whitelist;

ss.on("OverQuota", function(){
	while (ss.quotaUsage > 1)
		ss.storage.global_exclude_list.pop();
});


// Global rules

var global_settings = {
	include:"*",
	exclude:ss.storage.global_exclude_list,
	attachTo:["existing","top","frame"],
	contentStyleFile:d.url("css/common.css"),
	contentScriptFile:d.url('js/embeds.js')
};

var global_settings_js = {
	include:"*",
	exclude:ss.storage.global_exclude_list,
	attachTo:["existing","top"],
	contentScriptFile:d.url('js/common.js')
};

var global_mod = p.PageMod(global_settings);
var global_mod_js = p.PageMod(global_settings_js);


// Prepare rules

request({
	url:d.url('commons.json'),
	overrideMimeType: 'application/json',
	onComplete:function(r){
		commons = r.json;
	}
}).get();

request({
	url:d.url('rules.json'),
	overrideMimeType: 'application/json',
	onComplete:function(r){
		domains = r.json;
		
		for (var i in domains)
			custom_mods[i] = false;
	}
}).get();

request({
	url:d.url('blocking.json'),
	overrideMimeType: 'application/json',
	onComplete:function(r){
		block_urls = r.json;
	}
}).get();


// Activation functions

function activate_domain(hostname)
{
	if (typeof custom_mods[hostname] != 'undefined')
	{
		if (custom_mods[hostname] != false)
			return true;
		
		for (var i in ss.storage.global_exclude_list)
			if (ss.storage.global_exclude_list[i] == '*.'+hostname || ss.storage.global_exclude_list[i] == '*.www.'+hostname)
				return true;
		
		var mod = {
			include:'*.'+hostname,
			attachTo:["existing","top","frame"]
		};
		
		if (domains[hostname].s)
			mod.contentStyle = domains[hostname].s;
		else if (domains[hostname].c)
			mod.contentStyle = commons[domains[hostname].c];
		else
			mod.contentScriptFile = d.url('js/'+(domains[hostname].j > 0 ? 'common'+domains[hostname].j : hostname)+'.js');
		
		custom_mods[hostname] = p.PageMod(mod);
		return true;
	}
	
	return false;
}

function deactivate_domain(hostname)
{
	if (typeof custom_mods[hostname] != 'undefined')
	{
		if (custom_mods[hostname] == false)
			return true;
		
		custom_mods[hostname].destroy();
		custom_mods[hostname] = false;
		return true;
	}

	return false;
}

function toggle_activity(domain)
{
	domain = domain.replace(/^w{2,3}\d*\./i, '');
	
	var domain_rule = '*.'+domain;
	var deleted = 0;
	
	for (var i in ss.storage.global_exclude_list)
	{
		if (ss.storage.global_exclude_list[i] == domain_rule)
		{
			ss.storage.global_exclude_list.splice(i, 1);
			deleted = 1;
			break;
		}
	}
	
	if (deleted == 0)
		ss.storage.global_exclude_list[ss.storage.global_exclude_list.length] = domain_rule;
	
	
	global_mod.destroy();
	global_mod = p.PageMod(global_settings);
	
	global_mod_js.destroy();
	global_mod_js = p.PageMod(global_settings_js);
	
	
	// Rebuild custom rule if any
	if (deleted == 1)
	{
		if (!activate_domain(domain))
		{
			var possible_hosts = [];
			var host_parts = domain.split('.');
			
			for (var i=host_parts.length; i>=2; i--)
			{
				var part = host_parts.slice(-1*i).join('.');
				
				if (activate_domain(part))
					break;
			}
		}
	}
	
	// Destroy custom rule if any
	else
	{
		if (!deactivate_domain(domain))
		{
			var possible_hosts = [];
			var host_parts = domain.split('.');
			
			for (var i=host_parts.length; i>=2; i--)
			{
				var part = host_parts.slice(-1*i).join('.');
				
				if (deactivate_domain(part))
					break;
			}
		}
	}
}

function listener(event)
{
	var channel = event.subject.QueryInterface(Ci.nsIHttpChannel);
	
	if (/\.(jpg|jpeg|gif|png|ico)$/i.test(event.subject.URI.spec.split('?')[0]))
		return;
	
	var hostname = event.subject.URI.host.replace(/^w{2,3}\d*\./i, '');
	
	url_blocking(channel, hostname);
	
	if (activate_domain(hostname))
		return;
	
	var possible_hosts = [];
	var host_parts = hostname.split('.');
	
	for (var i=host_parts.length; i>=2; i--)
		if (activate_domain(host_parts.slice(-1*i).join('.')))
			return true;
};

function url_blocking(channel, hostname)
{
	for (var i in ss.storage.global_exclude_list)
		if (ss.storage.global_exclude_list[i] == '*.'+hostname || ss.storage.global_exclude_list[i] == '*.www.'+hostname)
			return true;
	
	var clean_url = channel.URI.spec.split('?')[0];
	
	
	// To shorten the checklist, many filters are grouped by keywords
	
	if (block_urls.common_groups)
	{
		for (var group in block_urls.common_groups)
		{
			if (channel.URI.spec.indexOf(group) > -1)
			{
				var group_filters = block_urls.common_groups[group];
				
				for (var i in group_filters)
				{
					if ((group_filters[i].q && channel.URI.spec.indexOf(group_filters[i].r) > -1) || (!group_filters[i].q && clean_url.indexOf(group_filters[i].r) > -1))
					{
						channel.cancel(Cr.NS_BINDING_ABORTED);
						return;
					}
				}
			}
		}
	}
	
	
	// Check ungrouped filters
	
	if (block_urls.common)
	{
		var group_filters = block_urls.common;
		
		for (var i in group_filters)
		{
			if ((group_filters[i].q && channel.URI.spec.indexOf(group_filters[i].r) > -1) || (!group_filters[i].q && clean_url.indexOf(group_filters[i].r) > -1))
			{
				channel.cancel(Cr.NS_BINDING_ABORTED);
				return;
			}
		}
	}
	
	
	// Site specific filters
	
	if (block_urls.specific)
	{
		for (var domain in block_urls.specific)
		{
			if (domain == hostname || hostname.endsWith('.'+domain))
			{
				for (var i in block_urls.specific[domain])
				{
					var rule = block_urls.specific[domain][i];
					
					if (channel.URI.spec.indexOf(rule) > -1)
					{
						channel.cancel(Cr.NS_BINDING_ABORTED);
						return;
					}
				}
			}
		}
	}
}


function setNotifications(options)
{
// 	if (options.loadReason === 'install')
// 		tabs.open("https://www.i-dont-care-about-cookies.eu");

	if (options.loadReason === 'upgrade')
	{
// 		tabs.open("https://www.i-dont-care-about-cookies.eu/whats-new/2019/?b=p");
		
// 		require("sdk/notifications").notify({
// 			title: "'I don't care about cookies' just got better",
// 			text: "324 websites added to the list! You'll see even fewer cookie warnings than before :)\n\nMake a small donation to support this project: click here to open its homepage.",
// 			onClick: function() {tabs.open("https://www.i-dont-care-about-cookies.eu/call-for-action/2018/");}
// 		});
	}
};

// Android

if (require("sdk/system").id == '{aa3c5121-dab2-40e2-81ca-7ea25febc110}')
{
	var {Cu} = require("chrome");
	Cu.import("resource://gre/modules/Services.jsm");
	Cu.import("resource://gre/modules/Prompt.jsm");

	var menuID = null;
	
	function onReport(window)
	{
		var p = new Prompt({
			window: window,
			title: "Please confirm",
			message: "Would you like to report a cookie warning on this website?",
			buttons: ["No", "Yes"]
		}).show(function(data) {
			
			var parts = tabs.activeTab.url.split('//');
			
			if (parts.length == 1)
				return;
			
			var domain = parts[1].split('/')[0].split(':')[0];
			
			if (domain.length > 0 && data.button == 1)
			{
				request({
					url: 'https://www.i-dont-care-about-cookies.eu/report/285.php?d=' + domain,
					onComplete: function (response) {
						window.NativeWindow.toast.show("Thanks! This website will probably be free of cookie warnings very soon :)", "long");
					}
				}).get();
			}
		});
	}
	
	function toggleMenu(visible)
	{
		let window = Services.wm.getMostRecentWindow("navigator:browser");
		let NativeWindow = window.NativeWindow;
		
		if (visible == false)
		{
			if (menuID)
			{
				NativeWindow.menu.remove(menuID);
				menuID = null;
			}
			
			return;
		}
		
		menuID = NativeWindow.menu.add({
			name:"I don't care about cookies"
		});
		
		NativeWindow.menu.add({
			name:"Report a cookie warning",
			parent:menuID,
			callback:function(){onReport(window);}
		}); 
		
		NativeWindow.menu.add({
			name:"Toggle on this domain",
			parent:menuID,
			callback:function(){
				var parts = tabs.activeTab.url.split('//');
				
				if (parts.length > 1)
					toggle_activity(parts[1].split('/')[0].split(':')[0]);
			}
		});
		
		NativeWindow.menu.add({
			name:"Support this project",
			parent:menuID,
			callback:function(){
				tabs.open("https://www.i-dont-care-about-cookies.eu/");
			}
		});
	}
	
	function androidContextMenuListener()
	{
		toggleMenu(prefs.prefs['contextmenu']);
	}
	
	exports.main = function(options)
	{
		events.on("http-on-modify-request", listener);
		toggleMenu(prefs.prefs['contextmenu']);
		prefs.on("contextmenu", androidContextMenuListener);
		setNotifications(options);
	};
	
	exports.onUnload = function(reason)
	{
		if (reason == 'disable' || reason == 'uninstall')
		{
			events.off("http-on-modify-request", listener);
			toggleMenu(false);
			prefs.off("contextmenu", androidContextMenuListener);
			delete ss.storage.global_exclude_list;
		}
	};
}


// Desktop

else
{
	// Context menu
	
	var cm = require("sdk/context-menu");
	var cm_menu;
	
	function toggleContextMenu(visible)
	{
		if (visible == false)
		{
			cm_menu.destroy();
			return;
		}
		
		cm_menu = cm.Menu({
			label:"I don't care about cookies",
			image:d.url("images/context-menu.png"),
			context:cm.SelectorContext("*"),
			items: [
				cm.Item({
					label:"Report a cookie warning",
					context:cm.URLContext(["http://*", "https://*"]),
					contentScriptFile:d.url("js/context-menu.js")
				}),
				
				cm.Item({
					label:"Disable on this domain",
					context:cm.PredicateContext(function(data){
						var parts = data.documentURL.split('://');
						
						if (parts.length > 1 && (parts[0] == 'http' || parts[0] == 'https'))
						{
							var domain = '*.'+parts[1].split('/')[0].replace(/^w{2,3}\d*\./i,"");
							
							for (var i in ss.storage.global_exclude_list)
								if (ss.storage.global_exclude_list[i] == domain)
									return false;
						
							return true;
						}
						
						return false;
					}),
					contentScript:
						'self.on("context",function(){return "Disable on "+window.location.hostname.replace(/^w{2,3}\d*\./i,"");});' +
						'self.on("click",function(){self.postMessage(window.location.hostname);});',
					onMessage:toggle_activity 
				}),
				
				cm.Item({
					label:"Enable on this domain",
					context:cm.PredicateContext(function(data){
						var parts = data.documentURL.split('://');
						
						if (parts.length > 1 && (parts[0] == 'http' || parts[0] == 'https'))
						{
							var domain = '*.'+parts[1].split('/')[0].replace(/^w{2,3}\d*\./i,"");
							
							for (var i in ss.storage.global_exclude_list)
								if (ss.storage.global_exclude_list[i] == domain)
									return true;
						}
						
						return false;
					}),
					contentScript:
						'self.on("context",function(){return "Enable on "+window.location.hostname.replace(/^w{2,3}\d*\./i,"");});' +
						'self.on("click",function(){self.postMessage(window.location.hostname);});',
					onMessage:toggle_activity 
				}),
				
				cm.Item({
					label:"Support this project",
					contentScript:'self.on("click",function(){self.postMessage("donate");});',
					onMessage:function(){
						tabs.open("https://www.i-dont-care-about-cookies.eu/");
					} 
				})
			]
		});
	}
	
	function contextMenuListener()
	{
		toggleContextMenu(prefs.prefs['contextmenu']);
	}
	
	exports.main = function(options)
	{
		events.on("http-on-modify-request", listener);
		toggleContextMenu(prefs.prefs['contextmenu']);
		prefs.on("contextmenu", contextMenuListener);
		setNotifications(options);
	};
	
	exports.onUnload = function(reason)
	{
		if (reason == 'disable' || reason == 'uninstall')
		{
			events.off("http-on-modify-request", listener);
			toggleContextMenu(false);
			prefs.off("contextmenu", contextMenuListener);
			delete ss.storage.global_exclude_list;
		}
	};
}